/*****************************************************************************
 * Copyright (c) 2005 StrutsBox.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    StrutsBox - initial API and implementation
 *****************************************************************************/
/*
 * $Id: StrutsConfigTranslator.java,v 1.4 2005/07/21 19:17:34 daniel_rohe Exp $
 */
package de.strutsbox.strutsconfig.translators;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.wst.common.internal.emf.resource.GenericTranslator;
import org.eclipse.wst.common.internal.emf.resource.IDTranslator;
import org.eclipse.wst.common.internal.emf.resource.RootTranslator;
import org.eclipse.wst.common.internal.emf.resource.Translator;

import de.strutsbox.strutsconfig.StrutsConfigPackage;

/**
 * The class <code>StrutsConfigTranslator</code> provides the root translator
 * that maps the StrutsConfig model to the Apache Struts configuration file.
 * 
 * @author Daniel
 */
public class StrutsConfigTranslator extends RootTranslator {

	/**
	 * shared instance
	 */
	private static StrutsConfigTranslator instance;

	/**
	 * Returns the shared instance of the translator.
	 * 
	 * @return shared instance
	 */
	public static StrutsConfigTranslator getInstance() {
		if (instance == null) {
			instance = new StrutsConfigTranslator();
		}
		return instance;
	}

	/**
	 * StrutsConfig package
	 */
	private static StrutsConfigPackage STRUTS_CONFIG_PACKAGE = StrutsConfigPackage.eINSTANCE;

	/**
	 * translator that maps the 'display-name' element to the 'displayName'
	 * attribute of the <code>Visualizable</code> class
	 */
	protected static Translator DISPLAY_NAME_TRANSLATOR = new Translator(
			StrutsConfigXmlMapping.DISPLAY_NAME, STRUTS_CONFIG_PACKAGE
					.getVisualizable_DisplayName());

	/**
	 * translator that maps the 'description' element to the 'description'
	 * attribute of the <code>Visualizable</code> class
	 */
	protected static Translator DESCRIPTION_TRANSLATOR = new Translator(
			StrutsConfigXmlMapping.DESCRIPTION, STRUTS_CONFIG_PACKAGE
					.getVisualizable_Description());

	/**
	 * translator that maps the 'smallIcon' element to the 'smallIcon' attribute
	 * of the <code>Visualizable</code> class
	 */
	protected static Translator SMALL_ICON_TRANSLATOR = new Translator(
			StrutsConfigXmlMapping.SMALL_ICON, STRUTS_CONFIG_PACKAGE
					.getVisualizable_SmallIcon());

	/**
	 * translator that maps the 'largeIcon' element to the 'largeIcon' attribute
	 * of the <code>Visualizable</code> class
	 */
	protected static Translator LARGE_ICON_TRANSLATOR = new Translator(
			StrutsConfigXmlMapping.LARGE_ICON, STRUTS_CONFIG_PACKAGE
					.getVisualizable_LargeIcon());

	/**
	 * Creates a translator that maps the 'set-property' element to the
	 * 'setProperty' reference of the <code>Configurable</code> class.
	 * 
	 * @return translator
	 */
	protected static Translator createSetPropertyTranslator() {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.SET_PROPERTY, STRUTS_CONFIG_PACKAGE
						.getConfigurable_SetProperty());
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.ATTR_PROPERTY,
						STRUTS_CONFIG_PACKAGE.getSetPropertyType_Property(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_VALUE,
						STRUTS_CONFIG_PACKAGE.getSetPropertyType_Value(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT) });
		return translator;
	}

	/**
	 * mappings for version 1.2
	 */
	private Translator[] children12;

	/**
	 * mappings for version 1.1
	 */
	private Translator[] children11;

	/**
	 * mappings for version 1.0
	 */
	private Translator[] children10;

	/**
	 * Creates a new <code>StrutsConfigTranslator</code>.
	 */
	public StrutsConfigTranslator() {
		super(StrutsConfigXmlMapping.STRUTS_CONFIG, STRUTS_CONFIG_PACKAGE
				.getStrutsConfigType());
	}

	/*
	 * @see org.eclipse.wst.common.internal.emf.resource.Translator#getChildren(java.lang.Object,
	 *      int)
	 */
	public Translator[] getChildren(Object target, int versionID) {
		switch (versionID) {
		case StrutsConfigConstants.VERSION_1_0_ID:
			if (children10 == null) {
				children10 = create10Children();
			}
			return children10;
		case StrutsConfigConstants.VERSION_1_1_ID:
			if (children11 == null) {
				children11 = create11Children();
			}
			return children11;
		case StrutsConfigConstants.VERSION_1_2_ID:
		default:
			if (children12 == null) {
				children12 = create12Children();
			}
			return children12;
		}
	}

	/**
	 * Creates the translators that map the children of the 'struts-config'
	 * element to the structural features of the <code>StrutsConfigType</code>
	 * class.
	 * <p>
	 * The translators are applicable for version 1.2.
	 * 
	 * @return translators
	 */
	protected Translator[] create12Children() {
		return new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.DISPLAY_NAME,
						STRUTS_CONFIG_PACKAGE.getStrutsConfigType_DisplayName()),
				new Translator(StrutsConfigXmlMapping.DESCRIPTION,
						STRUTS_CONFIG_PACKAGE.getStrutsConfigType_Description()),
				createDataSourcesTranslator(StrutsConfigConstants.VERSION_1_2_ID),
				createFormBeansTranslator(StrutsConfigConstants.VERSION_1_2_ID),
				createGlobalExceptionsTranslator(),
				createGlobalForwardsTranslator(StrutsConfigConstants.VERSION_1_2_ID),
				createActionMappingsTranslator(StrutsConfigConstants.VERSION_1_2_ID),
				createControllerTranslator(),
				createMessageResourcesTranslator(), createPlugInTranslator() };
	}

	/**
	 * Creates the translators that map the children of the 'struts-config'
	 * element to the structural features of the <code>StrutsConfigType</code>
	 * class.
	 * <p>
	 * The translators are applicable for version 1.1.
	 * 
	 * @return translators
	 */
	protected Translator[] create11Children() {
		return new Translator[] {
				IDTranslator.INSTANCE,
				createDataSourcesTranslator(StrutsConfigConstants.VERSION_1_1_ID),
				createFormBeansTranslator(StrutsConfigConstants.VERSION_1_1_ID),
				createGlobalExceptionsTranslator(),
				createGlobalForwardsTranslator(StrutsConfigConstants.VERSION_1_1_ID),
				createActionMappingsTranslator(StrutsConfigConstants.VERSION_1_1_ID),
				createControllerTranslator(),
				createMessageResourcesTranslator(), createPlugInTranslator() };
	}

	/**
	 * Creates the translators that map the children of the 'struts-config'
	 * element to the structural features of the <code>StrutsConfigType</code>
	 * class.
	 * <p>
	 * The translators are applicable for version 1.0.
	 * 
	 * @return translators
	 */
	protected Translator[] create10Children() {
		return new Translator[] {
				IDTranslator.INSTANCE,
				createDataSourcesTranslator(StrutsConfigConstants.VERSION_1_0_ID),
				createFormBeansTranslator(StrutsConfigConstants.VERSION_1_0_ID),
				createGlobalForwardsTranslator(StrutsConfigConstants.VERSION_1_0_ID),
				createActionMappingsTranslator(StrutsConfigConstants.VERSION_1_0_ID) };
	}

	/**
	 * Creates a translator that maps the 'data-sources' element to the
	 * 'dataSources' reference of the <code>StrutsConfigType</code> class.
	 * <p>
	 * The translator is applicable to version 1.0, 1.1 and 1.2.
	 * 
	 * @param versionID
	 *            version identifier
	 * @return translator
	 */
	protected Translator createDataSourcesTranslator(int versionID) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.DATA_SOURCES, STRUTS_CONFIG_PACKAGE
						.getStrutsConfigType_DataSources());
		translator.setChildren(new Translator[] { IDTranslator.INSTANCE,
				createDataSourceTranslator(versionID) });
		return translator;
	}

	/**
	 * Creates a translator that maps 'data-source' elements to the 'dataSource'
	 * reference of the <code>DataSourcesType</code> class.
	 * <p>
	 * The translator is applicable to version 1.0, 1.1 and 1.2.
	 * 
	 * @param versionID
	 *            version identifier
	 * @return translator
	 */
	protected Translator createDataSourceTranslator(int versionID) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.DATA_SOURCE, STRUTS_CONFIG_PACKAGE
						.getDataSourcesType_DataSource());
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.ATTR_KEY,
						STRUTS_CONFIG_PACKAGE.getDataSourceType_Key(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_TYPE,
						STRUTS_CONFIG_PACKAGE.getDataSourceType_Type(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				createSetPropertyTranslator() });

		// add mapping extensions from version 1.1 and 1.2
		if (StrutsConfigConstants.VERSION_1_1_ID == versionID
				|| StrutsConfigConstants.VERSION_1_2_ID == versionID) {
			Translator[] extension = new Translator[] { new Translator(
					StrutsConfigXmlMapping.ATTR_CLASS_NAME,
					STRUTS_CONFIG_PACKAGE.getDataSourceType_ClassName(),
					Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT) };
			GenericTranslator.appendChildren(translator, extension);
		}

		return translator;
	}

	/**
	 * Creates a translator that maps the 'form-beans' element to the
	 * 'formBeans' reference of the <code>StrutsConfigType</code> class.
	 * <p>
	 * The translator is applicable to version 1.0, 1.1 and 1.2.
	 * 
	 * @param versionID
	 *            version identifier
	 * @return translator
	 */
	protected Translator createFormBeansTranslator(int versionID) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.FORM_BEANS, STRUTS_CONFIG_PACKAGE
						.getStrutsConfigType_FormBeans());
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.ATTR_TYPE,
						STRUTS_CONFIG_PACKAGE.getFormBeansType_Type(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				createFormBeanTranslator(versionID) });
		return translator;
	}

	/**
	 * Creates a translator that maps 'form-bean' elements to the 'formBean'
	 * reference of the <code>FormBeansType</code> class.
	 * <p>
	 * The translator is applicable to version 1.0, 1.1 and 1.2.
	 * 
	 * @param versionID
	 *            version identifier
	 * @return translator
	 */
	protected Translator createFormBeanTranslator(int versionID) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.FORM_BEAN, STRUTS_CONFIG_PACKAGE
						.getFormBeansType_FormBean());
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.ATTR_CLASS_NAME,
						STRUTS_CONFIG_PACKAGE.getFormBeanType_ClassName(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_NAME,
						STRUTS_CONFIG_PACKAGE.getFormBeanType_Name(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_TYPE,
						STRUTS_CONFIG_PACKAGE.getFormBeanType_Type(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				SMALL_ICON_TRANSLATOR, LARGE_ICON_TRANSLATOR,
				DISPLAY_NAME_TRANSLATOR, DESCRIPTION_TRANSLATOR,
				createSetPropertyTranslator() });

		if (StrutsConfigConstants.VERSION_1_1_ID == versionID
				|| StrutsConfigConstants.VERSION_1_2_ID == versionID) {
			Translator[] extension = new Translator[] {
					new Translator(StrutsConfigXmlMapping.ATTR_DYNAMIC,
							STRUTS_CONFIG_PACKAGE.getFormBeanType_Dynamic(),
							Translator.DOM_ATTRIBUTE
									| Translator.BOOLEAN_LOWERCASE),
					createFormPropertyTranslator() };
			GenericTranslator.appendChildren(translator, extension);
		}

		return translator;
	}

	/**
	 * Creates a translator that maps 'form-property' elements to the
	 * 'formProperty' reference of the <code>FormBeanType</code> class.
	 * <p>
	 * The translator is applicable to version 1.1 and 1.2.
	 * 
	 * @return translator
	 */
	protected Translator createFormPropertyTranslator() {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.FORM_PROPERTY, STRUTS_CONFIG_PACKAGE
						.getFormBeanType_FormProperty());
		translator.setChildren(new Translator[] {
				new Translator(StrutsConfigXmlMapping.ATTR_CLASS_NAME,
						STRUTS_CONFIG_PACKAGE.getFormPropertyType_ClassName(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_INITIAL,
						STRUTS_CONFIG_PACKAGE.getFormPropertyType_Initial(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_NAME,
						STRUTS_CONFIG_PACKAGE.getFormPropertyType_Name(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_SIZE,
						STRUTS_CONFIG_PACKAGE.getFormPropertyType_Size(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_TYPE,
						STRUTS_CONFIG_PACKAGE.getFormPropertyType_Type(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				createSetPropertyTranslator() });
		return translator;
	}

	/**
	 * Creates a translator that maps the 'global-exceptions' element to the
	 * 'globalExceptions' reference of the <code>StrutsConfigType</code>
	 * class.
	 * <p>
	 * The translator is applicable to version 1.1 and 1.2.
	 * 
	 * @return translator
	 */
	protected Translator createGlobalExceptionsTranslator() {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.GLOBAL_EXCEPTIONS, STRUTS_CONFIG_PACKAGE
						.getStrutsConfigType_GlobalExceptions());
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				createExceptionTranslator(STRUTS_CONFIG_PACKAGE
						.getGlobalExceptionsType_Exception()) });
		return translator;
	}

	/**
	 * Creates a translator that maps 'exception' elements to the given
	 * structural feature.
	 * <p>
	 * The translator is applicable to version 1.1 and 1.2.
	 * 
	 * @param feature
	 *            structural feature
	 * @return translator
	 */
	protected Translator createExceptionTranslator(EStructuralFeature feature) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.EXCEPTION, feature);
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.ATTR_BUNDLE,
						STRUTS_CONFIG_PACKAGE.getExceptionType_Bundle(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_CLASS_NAME,
						STRUTS_CONFIG_PACKAGE.getExceptionType_ClassName(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_HANDLER,
						STRUTS_CONFIG_PACKAGE.getExceptionType_Handler(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_KEY,
						STRUTS_CONFIG_PACKAGE.getExceptionType_Key(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_PATH,
						STRUTS_CONFIG_PACKAGE.getExceptionType_Path(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				new Translator(StrutsConfigXmlMapping.ATTR_SCOPE,
						STRUTS_CONFIG_PACKAGE.getExceptionType_Scope(),
						Translator.DOM_ATTRIBUTE),
				new Translator(StrutsConfigXmlMapping.ATTR_TYPE,
						STRUTS_CONFIG_PACKAGE.getExceptionType_Type(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				SMALL_ICON_TRANSLATOR, LARGE_ICON_TRANSLATOR,
				DISPLAY_NAME_TRANSLATOR, DESCRIPTION_TRANSLATOR,
				createSetPropertyTranslator() });
		return translator;
	}

	/**
	 * Creates a translator that maps the 'global-forwards' element to the
	 * 'globalForwards' reference of the <code>StrutsConfigType</code> class.
	 * <p>
	 * The translator is applicable to version 1.0, 1.1 and 1.2.
	 * 
	 * @param versionID
	 *            version identifier
	 * @return translator
	 */
	protected Translator createGlobalForwardsTranslator(int versionID) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.GLOBAL_FORWARDS, STRUTS_CONFIG_PACKAGE
						.getStrutsConfigType_GlobalForwards());
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.ATTR_TYPE,
						STRUTS_CONFIG_PACKAGE.getGlobalForwardsType_Type(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				createForwardTranslator(versionID, STRUTS_CONFIG_PACKAGE
						.getGlobalForwardsType_Forward()) });
		return translator;
	}

	/**
	 * Creates a translator that maps 'forward' elements to the given structural
	 * feature.
	 * 
	 * @param versionID
	 *            version identifier
	 * @param feature
	 *            structural feature
	 * @return translator
	 */
	protected Translator createForwardTranslator(int versionID,
			EStructuralFeature feature) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.FORWARD, feature);
		translator
				.setChildren(new Translator[] {
						IDTranslator.INSTANCE,
						new Translator(StrutsConfigXmlMapping.ATTR_CLASS_NAME,
								STRUTS_CONFIG_PACKAGE
										.getForwardType_ClassName(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_NAME,
								STRUTS_CONFIG_PACKAGE.getForwardType_Name(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_PATH,
								STRUTS_CONFIG_PACKAGE.getForwardType_Path(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_REDIRECT,
								STRUTS_CONFIG_PACKAGE.getForwardType_Redirect(),
								Translator.DOM_ATTRIBUTE
										| Translator.BOOLEAN_LOWERCASE),
						SMALL_ICON_TRANSLATOR, LARGE_ICON_TRANSLATOR,
						DISPLAY_NAME_TRANSLATOR, DESCRIPTION_TRANSLATOR,
						createSetPropertyTranslator() });

		// add mapping extensions from version 1.1 and 1.2
		if (StrutsConfigConstants.VERSION_1_1_ID == versionID
				|| StrutsConfigConstants.VERSION_1_2_ID == versionID) {
			GenericTranslator.appendChild(translator, new Translator(
					StrutsConfigXmlMapping.ATTR_CONTEXT_RELATIVE,
					STRUTS_CONFIG_PACKAGE.getForwardType_ContextRelative(),
					Translator.DOM_ATTRIBUTE | Translator.BOOLEAN_LOWERCASE));
		}

		// add mapping extensions from version 1.2
		if (StrutsConfigConstants.VERSION_1_2_ID == versionID) {
			GenericTranslator.appendChild(translator, new Translator(
					StrutsConfigXmlMapping.ATTR_MODULE, STRUTS_CONFIG_PACKAGE
							.getForwardType_Module(), Translator.DOM_ATTRIBUTE
							| Translator.CDATA_CONTENT));
		}

		return translator;
	}

	/**
	 * Creates a translator that maps the 'action-mappings' element to the
	 * 'actionMappings' reference of the <code>StrutsConfigType</code> class.
	 * <p>
	 * The translator is applicable to version 1.0, 1.1 and 1.2.
	 * 
	 * @param versionID
	 *            version identifier
	 * @return translator
	 */
	protected Translator createActionMappingsTranslator(int versionID) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.ACTION_MAPPINGS, STRUTS_CONFIG_PACKAGE
						.getStrutsConfigType_ActionMappings());
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.ATTR_TYPE,
						STRUTS_CONFIG_PACKAGE.getActionMappingsType_Type(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				createActionTranslator(versionID) });

		return translator;
	}

	/**
	 * Creates a translator that maps 'action' elements to the 'action'
	 * reference of the <code>ActionMappingsType</code> class.
	 * <p>
	 * The translator is applicable for version 1.0, 1.1 and 1.2.
	 * 
	 * @param versionId
	 *            version identifier
	 * @return translator
	 */
	protected Translator createActionTranslator(int versionID) {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.ACTION, STRUTS_CONFIG_PACKAGE
						.getActionMappingsType_Action());
		translator
				.setChildren(new Translator[] {
						IDTranslator.INSTANCE,
						new Translator(
								StrutsConfigXmlMapping.ATTR_ATTRIBUTE,
								STRUTS_CONFIG_PACKAGE.getActionType_Attribute(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_CLASS_NAME,
								STRUTS_CONFIG_PACKAGE.getActionType_ClassName(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_FORWARD,
								STRUTS_CONFIG_PACKAGE.getActionType_Forward1(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_INCLUDE,
								STRUTS_CONFIG_PACKAGE.getActionType_Include(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_INPUT,
								STRUTS_CONFIG_PACKAGE.getActionType_Input(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_NAME,
								STRUTS_CONFIG_PACKAGE.getActionType_Name(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_PARAMETER,
								STRUTS_CONFIG_PACKAGE.getActionType_Parameter(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_PATH,
								STRUTS_CONFIG_PACKAGE.getActionType_Path(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_PREFIX,
								STRUTS_CONFIG_PACKAGE.getActionType_Prefix(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_SCOPE,
								STRUTS_CONFIG_PACKAGE.getActionType_Scope(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_SUFFIX,
								STRUTS_CONFIG_PACKAGE.getActionType_Suffix(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_TYPE,
								STRUTS_CONFIG_PACKAGE.getActionType_Type(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_UNKNOWN,
								STRUTS_CONFIG_PACKAGE.getActionType_Unknown(),
								Translator.DOM_ATTRIBUTE
										| Translator.BOOLEAN_LOWERCASE),
						new Translator(
								StrutsConfigXmlMapping.ATTR_VALIDATE,
								STRUTS_CONFIG_PACKAGE.getActionType_Attribute(),
								Translator.DOM_ATTRIBUTE
										| Translator.BOOLEAN_LOWERCASE),
						SMALL_ICON_TRANSLATOR,
						LARGE_ICON_TRANSLATOR,
						DISPLAY_NAME_TRANSLATOR,
						DESCRIPTION_TRANSLATOR,
						createSetPropertyTranslator(),
						createForwardTranslator(versionID,
								STRUTS_CONFIG_PACKAGE.getActionType_Forward()) });

		// add mapping extensions from version 1.1 and 1.2
		if (StrutsConfigConstants.VERSION_1_1_ID == versionID
				|| StrutsConfigConstants.VERSION_1_2_ID == versionID) {
			Translator[] extension = new Translator[] {
					new Translator(StrutsConfigXmlMapping.ATTR_ROLES,
							STRUTS_CONFIG_PACKAGE.getActionType_Roles(),
							Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
					createExceptionTranslator(STRUTS_CONFIG_PACKAGE
							.getActionType_Exception()) };
			GenericTranslator.appendChildren(translator, extension);
		}

		return translator;
	}

	/**
	 * Creates a translator that maps the 'controller' element to the
	 * 'controller' reference of the <code>StrutsConfigType</code> class.
	 * <p>
	 * The translator is applicable for version 1.1 and 1.2.
	 * 
	 * @return translator
	 */
	protected Translator createControllerTranslator() {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.CONTROLLER, STRUTS_CONFIG_PACKAGE
						.getStrutsConfigType_Controller());
		translator
				.setChildren(new Translator[] {
						IDTranslator.INSTANCE,
						new Translator(StrutsConfigXmlMapping.ATTR_BUFFER_SIZE,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_BufferSize(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_CLASS_NAME,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_ClassName(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_CONTENT_TYPE,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_ContentType(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_FORWARD_PATTERN,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_ForwardPattern(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_INPUT_FORWARD,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_InputForward(),
								Translator.DOM_ATTRIBUTE
										| Translator.BOOLEAN_LOWERCASE),
						new Translator(StrutsConfigXmlMapping.ATTR_LOCALE,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_Locale(),
								Translator.DOM_ATTRIBUTE
										| Translator.BOOLEAN_LOWERCASE),
						new Translator(
								StrutsConfigXmlMapping.ATTR_MAX_FILE_SIZE,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_MaxFileSize(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_MEM_FILE_SIZE,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_MemFileSize(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_MULTIPART_CLASS,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_MultipartClass(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_NOCACHE,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_Nocache(),
								Translator.DOM_ATTRIBUTE
										| Translator.BOOLEAN_LOWERCASE),
						new Translator(
								StrutsConfigXmlMapping.ATTR_PAGE_PATTERN,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_PagePattern(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(
								StrutsConfigXmlMapping.ATTR_PROCESSOR_CLASS,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_ProcessorClass(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_TEMP_DIR,
								STRUTS_CONFIG_PACKAGE
										.getControllerType_TempDir(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						createSetPropertyTranslator() });
		return translator;
	}

	/**
	 * Creates a translator that maps 'message-resources' elements to the
	 * 'messageResources' reference of the <code>StrutsConfigType</code>
	 * class.
	 * <p>
	 * The translator is applicable for version 1.1 and 1.2.
	 * 
	 * @return translator
	 */
	protected Translator createMessageResourcesTranslator() {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.MESSAGE_RESOURCES, STRUTS_CONFIG_PACKAGE
						.getStrutsConfigType_MessageResources());
		translator
				.setChildren(new Translator[] {
						IDTranslator.INSTANCE,
						new Translator(StrutsConfigXmlMapping.ATTR_CLASS_NAME,
								STRUTS_CONFIG_PACKAGE
										.getMessageResourcesType_ClassName(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_FACTORY,
								STRUTS_CONFIG_PACKAGE
										.getMessageResourcesType_Factory(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_KEY,
								STRUTS_CONFIG_PACKAGE
										.getMessageResourcesType_Key(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						new Translator(StrutsConfigXmlMapping.ATTR_NULL,
								STRUTS_CONFIG_PACKAGE
										.getMessageResourcesType_Null(),
								Translator.DOM_ATTRIBUTE
										| Translator.BOOLEAN_LOWERCASE),
						new Translator(StrutsConfigXmlMapping.ATTR_PARAMETER,
								STRUTS_CONFIG_PACKAGE
										.getMessageResourcesType_Parameter(),
								Translator.DOM_ATTRIBUTE
										| Translator.CDATA_CONTENT),
						createSetPropertyTranslator() });
		return translator;
	}

	/**
	 * Creates the translator that maps 'plug-in' elements to the 'plugIn'
	 * reference of the <code>StrutsConfigType</code> class.
	 * <p>
	 * The translator is applicable for version 1.1 and 1.2.
	 * 
	 * @return translator
	 */
	protected Translator createPlugInTranslator() {
		GenericTranslator translator = new GenericTranslator(
				StrutsConfigXmlMapping.PLUG_IN, STRUTS_CONFIG_PACKAGE
						.getStrutsConfigType_PlugIn());
		translator.setChildren(new Translator[] {
				IDTranslator.INSTANCE,
				new Translator(StrutsConfigXmlMapping.ATTR_CLASS_NAME,
						STRUTS_CONFIG_PACKAGE.getPlugInType_ClassName(),
						Translator.DOM_ATTRIBUTE | Translator.CDATA_CONTENT),
				createSetPropertyTranslator() });
		return translator;
	}

}
