/*****************************************************************************
 * Copyright (c) 2005 StrutsBox.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    StrutsBox - initial API and implementation
 *****************************************************************************/
/*
 * $Id: StrutsConfigUtil.java,v 1.3 2005/07/21 15:21:01 daniel_rohe Exp $
 */
package de.strutsbox.strutsconfig.util;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.resource.Resource;

/**
 * The class <code>StrutsConfigUtil</code> provides some utility methods for
 * working with the StrutsConfig model.
 * 
 * @author Daniel
 */
public class StrutsConfigUtil {

	/**
	 * Returns the first occurence of an object of the given type in the given
	 * resource.
	 * 
	 * @param resource
	 *            resource where to find
	 * @param type
	 *            object type
	 * @return object or <code>null</code>
	 */
	public static Object getObjectByType(Resource resource, EClassifier type) {
		TreeIterator iter = resource.getAllContents();
		return getObjectByType(iter, type);
	}

	/**
	 * Returns the first occurence of an object of the given type in the given
	 * iterator.
	 * 
	 * @param iter
	 *            iterator where to find
	 * @param type
	 *            object type
	 * @return object or <code>null</code>
	 */
	public static Object getObjectByType(TreeIterator iter, EClassifier type) {
		while (iter.hasNext()) {
			Object element = (Object) iter.next();
			if (type.isInstance(element)) {
				return element;
			}
		}
		return null;
	}

	/**
	 * Returns the occurences of objects of the given type in the given
	 * resource.
	 * 
	 * @param resource
	 *            resource where to find
	 * @param type
	 *            object type
	 * @return collection of objects
	 */
	public static Collection getObjectsByType(Resource resource,
			EClassifier type) {
		TreeIterator iter = resource.getAllContents();
		return getObjectsByType(iter, type);
	}

	/**
	 * Returns the occurences of objects of the given type in the given
	 * iterator.
	 * 
	 * @param iter
	 *            iterator where to find
	 * @param type
	 *            object type
	 * @return collection of objects
	 */
	public static Collection getObjectsByType(TreeIterator iter,
			EClassifier type) {
		Collection result = new ArrayList();
		while (iter.hasNext()) {
			Object element = (Object) iter.next();
			if (type.isInstance(element)) {
				result.add(element);
			}
		}
		return result;
	}

	/**
	 * Creates a new <code>StrutsConfigUtil</code>.
	 */
	private StrutsConfigUtil() {
		super();
	}

}
